<?php

  /**
   * Class used to communicate with the Riya API. It contains the
   * necessary code to handle the most common errors and also helps
   * with the XML.
   * 
   * The callMethod() method returns a SimpleXMLElement object which
   * then has to be parsed by the user.
   * 
   * Requirements: PHP 5.0 (curl for image uploads)
   */
  class RiyaApi {
    
    /**
     * URI to the Riya API
     *
     * @var string
     */
    const URI = 'http://www.riya.com/rest';
    
    /**
     * API Key
     *
     * @var string
     */
    protected $apiKey;
    
    /**
     * Riya Username
     *
     * @var string
     */
    protected $username;
    
    /**
     * Auth token
     *
     * @var string
     */
    protected $authToken;
    
    /**
     * Contains the an error code. If the error is not from the remote
     * server it will only contain 0, and errorMessage will instead
     * contain a proper description of the error.
     *
     * @var integer
     */
    protected $errorCode;
    
    /**
     * Contains a short error description which can be used to debug.
     *
     * @var string
     */
    protected $errorMessage;
    
    /**
     * Sets the API key. An API key can be generated on 
     * http://www.riya.com/riyaAPI
     *
     * @param string $apiKey
     * @return void
     */
    public function setApiKey($apiKey) {
      $this->apiKey = (string)$apiKey;
    }
    
    /**
     * Sets username. This is required to communicate with the API. 
     *
     * @param string $username
     * @return void
     */
    public function setUsername($username) {
      $this->username = (string)$username;
    }
    
    /**
     * Sets auth token. This is required to use any method. A new token
     * can be retrieved using the fetchAuthToken() method.
     * 
     * @param string $token
     * @return void
     */
    public function setAuthToken($token) {
      $this->authToken = $token;
    }
    
    /**
     * 
     *
     * @param string $password
     * @return string
     */
    public function getAuthToken($password) {
      // Call the local callMethod
      $xmlObj = $this->callMethod('riya.auth.GetToken', array('password' => $password));

      if ( $xmlObj === false ) {
        // Don't set a new error here message here. If we do so it will
        // overwrite the one set in the callMethod() method.
        return false;
      }
      
      return (string)$xmlObj->auth->token;
    }
    
    /**
     * Communicates with the API. When trying to upload a photo it
     * makes use of curl, otherwise it uses file_get_contents() and
     * should be compatible with the majority of the webhosts. 
     * Returns false if something fails.
     *
     * @param string $method
     * @param array $args
     * @return SimpleXMLElement
     */
    public function callMethod($method, $args = null) {
      
      // Make sure args is an array
      if ( is_null($args) || !is_array($args) ) {
        $args = array();
      }
      
      // Handle arguments
      $args['api_key']   = $this->apiKey;
      $args['method']    = $method;
      
      if ( !is_null($this->username) ) {
        $args['user_name'] = $this->username;
      }
      
      if ( !is_null($this->authToken) ) {
        $args['auth_token'] = $this->authToken;
      }
      
      // Determine if we have an upload, then use curl
      if ( strtolower($method) == 'riya.photos.upload.uploadphoto' ) {
        
        // Requies curl
        if ( !extension_loaded('curl') ) {
          $this->errorCode = 0;
          $this->errorMessage = 'riya.photos.upload.UploadPhoto requires cURL to be enabled. Please see www.php.net/curl for more information.';
          return false;
        }
        
        // File must be readable
        if ( !isset($args['photo']) || !is_readable($args['photo']) || is_dir($args['photo']) ) {
          $this->errorCode = 0;
          $this->errorMessage = 'You have not set a photo to upload,  or the photo you have chosen to upload was not readable. Path: ' . $args['photo'];
          return false;
        }
        
        // Prepend with @ for cURL to understand it's a file
        $args['photo'] = '@' . $args['photo'];
        
        // Init and send post requests
        $ch = curl_init(self::URI);  
        curl_setopt($ch, CURLOPT_POSTFIELDS,     $args);
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
        
        $data = curl_exec($ch);
        
        // Close curl
        curl_close($ch);
        
      } else {
      
        // Construct the URI to call
        $callUri = self::URI . '?' . http_build_query($args, null, '&');
        
        // Use file_get_contents() to try to retrieve the URI. This might
        // fail but spits out ugly warnings so we use the internal error
        // handling. -- skip file_get_contents for cURL
//        if ( ($data = @file_get_contents($callUri)) === false ) {
//          $this->errorCode = 0;
//          $this->errorMessage = 'file_get_contents() failed. Unable to get the following URI: ' . $callUri;
//          return false;
//        }

        $ch = curl_init($callUri);
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
        $data = curl_exec($ch);
      
      }
      
      // Try to create a SimpleXMLElement object
      $xmlObj = @simplexml_load_string($data);
      
      // If the above failed set an error, and return false
      if ( $xmlObj === false ) {
        $this->errorCode = 0;
        $this->errorMessage = 'The server response contained malformed XML. Data: ' . $data;
        return false;
      }
      
      // Check for server-side errors
      $status = self::parseAttributes($xmlObj);
      
      // If we have a server error, return false
      if ( $status['stat'] == 'fail' ) {
        $errors = self::parseAttributes($xmlObj->error);
        $this->errorCode = $errors['code'];
        $this->errorMessage = $errors['msg'];
        return false;
      }
      
      return $xmlObj;
    }
    
    /**
     * Return error code. (0 = not from the remote server)
     *
     * @return integer
     */
    public function getErrorCode() {
      return $this->errorCode;
    }
    
    /**
     * Return error message.
     *
     * @return string
     */
    public function getErrorMessage() {
      return $this->errorMessage;
    }
    
    /**
     * A generic method to parse the attributes of a tag and make it an
     * array.
     *
     * @param SimpleXMLElement $obj
     * @return array
     */
    public static function parseAttributes($obj) {
      $arr = array();
      foreach ($obj->attributes() as $name => $value) {
        $arr[$name] = (string)$value;
      }
      return $arr;
    }
    
  }
  
?>